-- Support for MIME attachments

require 'split'

xds_b.xop = {}
xds_b.xop.documentInclude = {}
xds_b.xop.attachments = {}
xds_b.xop.soap = {}

local addElement = node.addElement
local setAttr = node.setAttr

local function buildMimeHeader(ContentUUID, ContentType, Index)
   local MimeHeader = 'Content-Type: CONTENTTYPE\r\nContent-Transfer-Encoding: binary\r\nContent-ID: <INDEX.urn:uuid:CONTENTUUID>\r\n\r\n'
   MimeHeader = MimeHeader:gsub('CONTENTUUID', ContentUUID)   
   MimeHeader = MimeHeader:gsub('INDEX', Index)
   MimeHeader = MimeHeader:gsub('CONTENTTYPE', ContentType)
   
   return MimeHeader
end

local function buildMimeBoundary(MimeUUID)
   local MimeBoundary = '--MIMEBoundaryurn_uuid_MIMEUUID'   
   return MimeBoundary:gsub('MIMEUUID', MimeUUID)
end

local function getMimeBoundary(Response)
   if Response:find('--MIMEBoundaryurn_uuid_[%d%a]*') then
      return Response:sub(Response:find('--MIMEBoundaryurn_uuid_[%d%a]*'))
   else
      return nil
   end
end

-- xds_b.xop interface

local DocumentIncludeAddHelp = {
   title="xds_b.xop.documentInclude.add",
   usage=[[xds_b.xop.documentInclude.add{target=<root tag>, documentEntryId=<value>, documentUUID=<value>, 
      index=<value>}]],
   description="This function will add a document include construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The root xml where the document include XML will be inserted",
      documentEntryId="The document entry Id",
      documentUUID = "The UUID associated with the attached document",
      index="The index into the attachment set that this document include refers to"
   },
   examples={[[local DocumentInclude = xds_b.xop.documentInclude.add{target=Root, 
      documentEntryId='DocumentEntryId1', documentUUID=7F62ED515B03490A7CC001953D983C15, index=1}]]}
}

function xds_b.xop.documentInclude.add(I)
   ihe.help.check(I, DocumentIncludeAddHelp)
   
   local DocumentEntryId = I.documentEntryId
   local DocumentUUID = I.documentUUID
   local Index = I.index
   
   local REF = addElement(I.target, 'xdsb:Document')
   setAttr(REF, 'id', DocumentEntryId)   
   local INC = addElement(REF, 'xop:Include')
   setAttr(INC, 'href', 'cid:'..Index..'.urn:uuid:'..DocumentUUID)
   setAttr(INC, 'xmlns:xop', 'http://www.w3.org/2004/08/xop/include') 
   
   return REF
end

ihe.help.set{func=xds_b.xop.documentInclude.add, info=DocumentIncludeAddHelp}

local AttachmentsAddHelp = {
   title="xds_b.xop.attachments.add",
   usage="xds_b.xop.attachments.add{attachments=<values>, content=<value>, mime=<value>, soap=<value>}",
   description="This function will add a attachments to the inputted SOAP request.",
   returns={
      "The string representing the resulting SOAP request"
   },
   parameters={
      attachments="The table of documents to attach",
      content="The content UUID used to build the request header",
      mime="The MIME UUID used to build the MIME boundaries",
      soap="The base Provide and Register Document Set request"
   },
   examples={[[ local Request = xds_b.xop.attachments.add{attachments={{documentId='DocumentId1', 
      document='Hello World!', documentMimeType='text/plain', documentUUID=xds_b.getUUID()}, 
      content=xds_b.getUUID(), mime=xds_b.getUUID(), soap=BaseSoap}]]}
}

function xds_b.xop.attachments.add(I)
   ihe.help.check(I, AttachmentsAddHelp)
   
   local Attachments = I.attachments
   local MimeUUID = I.mime
   local ContentUUID = I.content
   local Soap = I.soap
   
   local PackagedBody = ''
   PackagedBody = PackagedBody
   ..buildMimeBoundary(MimeUUID)..'\r\n'
   ..buildMimeHeader(ContentUUID, 'application/xop+xml; charset=UTF-8; type=\'application/soap+xml\'', 0)
   ..'<?xml version="1.0" encoding="UTF-8"?>'
   ..tostring(Soap):gsub('\n', ''):gsub('>%s+<', '><')..'\r\n'
   
   if nil ~= Attachments then
      for i = 1, #Attachments do
         local Attachment = Attachments[i]
         PackagedBody = PackagedBody
         ..buildMimeBoundary(MimeUUID)..'\r\n'
         ..buildMimeHeader(Attachment.uuid, Attachment.mime, i)
         ..Attachment.content..'\r\n'
      end
      
      PackagedBody = PackagedBody..'\r\n'..buildMimeBoundary(MimeUUID)..'--'
   else
      PackagedBody = PackagedBody..'\r\n'..buildMimeBoundary(MimeUUID)..'--'
   end
   
   return PackagedBody
end

ihe.help.set{func=xds_b.xop.attachments.add, info=AttachmentsAddHelp}

local AttachmentsExtractHelp = {
   title="xds_b.xop.attachments.extract",
   usage="xds_b.xop.attachments.extract{response=<values>}",
   description="This function will extract attachments from a SOAP response.",
   returns={
      "The table of attached documents extracted from the SOAP response"
   },
   parameters={
      response="The response containing the attachments"
   },
   examples={[[ local Attachments = xds_b.xop.attachments.extract{response=Response}]]}
}

function xds_b.xop.attachments.extract(I)
   ihe.help.check(I, AttachmentsExtractHelp)
   
   local Response = I.response
   
   local DocumentList = {}

   local MimeContents = Response:split(getMimeBoundary(Response))
      
   for i = 3, #MimeContents do
      _, DocumentList[i - 2] = pcall(function() return MimeContents[i]:split('\r\n\r\n')[2] end)
   end
      
   return DocumentList
end

ihe.help.set{func=xds_b.xop.attachments.extract, info=AttachmentsExtractHelp}

local SoapExtractHelp = {
   title="xds_b.soap.extract",
   usage="xds_b.soap.extract{response=<value>}",
   description="This function will extract the SOAP request from a request with attachments.",
   returns={
      "The string representing the SOAP message"
   },
   parameters={
      response="The SOAP request with attachments"
   },
   examples={"local Soap = xds_b.soap.extract{response=Response}"}
}

function xds_b.xop.soap.extract(I)
   ihe.help.check(I, SoapExtractHelp)
   
   local Response = I.response
   
   return pcall(function() return Response:split(getMimeBoundary(Response))[2]:split('\r\n\r\n')[2] end)
end

ihe.help.set{func=xds_b.xop.soap.extract, info=SoapExtractHelp}